package com.yerihyo.yeritools;

import java.beans.BeanInfo;
import java.beans.IntrospectionException;
import java.beans.Introspector;
import java.beans.PropertyDescriptor;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.OutputStream;
import java.io.Serializable;
import java.lang.reflect.Method;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.jar.JarEntry;
import java.util.jar.JarInputStream;


public class RTTIToolkit {
	public static Object deserialize(byte[] model)
			throws ClassNotFoundException, IOException {
		return readObject(new ByteArrayInputStream(model));
	}
	
	public static byte[] serialize(Serializable o){
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		try {
			writeObject(baos, o);
			baos.close();
		} catch (IOException e) {
			throw new RuntimeException(e);
		}
		return baos.toByteArray();
	}
	
	public static Object readObject(InputStream in) throws IOException, ClassNotFoundException  {
		ObjectInputStream ois = new ObjectInputStream(in);
		Object object = ois.readObject();
		ois.close();
		return object;
	}

	public static void writeObject(OutputStream out, Serializable o) throws IOException {
		ObjectOutputStream oos = new ObjectOutputStream(out);
		oos.writeObject(o);
		oos.close();
	}
	
	public static List<PropertyDescriptor> getWekaProperties(
			Class<?> targetClass) {
		ArrayList<PropertyDescriptor> returnValue = new ArrayList<PropertyDescriptor>();

		PropertyDescriptor[] properties;
		try {
			BeanInfo bi = Introspector.getBeanInfo(targetClass);
			properties = bi.getPropertyDescriptors();
		} catch (IntrospectionException ex) {
			return returnValue;
		}

		for (int i = 0; i < properties.length; i++) {
			// Don't display hidden or expert properties.
			if (properties[i].isHidden() || properties[i].isExpert()) {
				continue;
			}

			if (properties[i].getDisplayName().equals("options"))
				continue;

			Method getter = properties[i].getReadMethod();
			Method setter = properties[i].getWriteMethod();

			// Only display read/write properties.
			if (getter == null || setter == null) {
				continue;
			}

			returnValue.add(properties[i]);
		}
		return returnValue;
	}

	public static List<PropertyDescriptor> getsetProperties(Class<?> targetClass) {
		ArrayList<PropertyDescriptor> returnValue = new ArrayList<PropertyDescriptor>();

		PropertyDescriptor[] properties;
		try {
			BeanInfo bi = Introspector.getBeanInfo(targetClass);
			properties = bi.getPropertyDescriptors();
		} catch (IntrospectionException ex) {
			return returnValue;
		}

		for (int i = 0; i < properties.length; i++) {
			// Don't display hidden or expert properties.
			if (properties[i].isHidden() || properties[i].isExpert()) {
				continue;
			}

			if (properties[i].getDisplayName().equals("options"))
				continue;

			Method getter = properties[i].getReadMethod();
			Method setter = properties[i].getWriteMethod();

			// Only display read/write properties.
			if (getter == null || setter == null) {
				continue;
			}

			returnValue.add(properties[i]);
		}
		return returnValue;
	}

	public static String getPropertyDescription(PropertyDescriptor property) {
		String name = property.getDisplayName();
		Class<?> type = property.getPropertyType();
		return type.getName() + " " + name;
	}

	public static String makeXMLOption(Object o) throws IllegalAccessException,
			InstantiationException, ClassNotFoundException {
		if (o instanceof Boolean || o instanceof Integer || o instanceof Double) {
			return o.toString();
		} else if (o instanceof java.lang.String) {
			return (String) o;
		} else {
			return o.getClass().getName();
		}
	}

	public static Object getValueInType(Class<?> type, String value)
			throws IllegalAccessException, InstantiationException,
			ClassNotFoundException {
		if (type == boolean.class) {
			return Boolean.parseBoolean(value);
		} else if (type == int.class) {
			return Integer.parseInt(value);
		} else if (type == double.class) {
			return Double.parseDouble(value);
		} else if (type == java.lang.String.class) {
			return value;
		}

		// there is one dimension array in weka until now.
		else if (type.isArray()) {
			ArrayList<Object> returnValue = new ArrayList<Object>();

			return returnValue.toArray();
		} else {
			return Class.forName(value).newInstance();
		}
	}

	public static String objectToStringByType(Object o)
			throws IllegalAccessException, InstantiationException,
			ClassNotFoundException {
		if (o instanceof Boolean || o instanceof Integer || o instanceof Double) {
			return o.toString();
		} else if (o instanceof java.lang.String) {
			return (String) o;
		} else {
			return o.getClass().getName();
		}
	}

	public static void fill(Object target, Map<String, String> map)
			throws Exception {
		List<PropertyDescriptor> properties = getsetProperties(target
				.getClass());

		for (PropertyDescriptor property : properties) {

			String name = property.getDisplayName();
			Class<?> type = property.getPropertyType();
			Method setter = property.getWriteMethod();

			Object value = getValueInType(type, map.get(name));
			setter.invoke(target, value);
		}
	}

	public static String arrayToString(Object o) {
		if (!o.getClass().isArray()) {
			return o.toString();
		}
		StringBuffer returnValue = new StringBuffer();
		returnValue.append('[');
		for (Object obj : (Object[]) o) {
			returnValue.append(arrayToString(obj));
		}
		returnValue.append(']');
		return returnValue.toString();
	}

	/**
	 * ArrayString class
	 * 
	 * @author Moonyoung
	 * 
	 */
	public static class ArrayString {
		private String arrayStartString;
		private String arrayEndString;
		private String deliminator;

		public ArrayString() throws StringNotInArrayFormException {
			this("{", "}", ",");
		}

		public ArrayString(String arrayStartString, String arrayEndString,
				String deliminator) throws StringNotInArrayFormException {
			setArrayStartString(arrayStartString);
			setArrayEndString(arrayEndString);
			setDeliminator(deliminator);
		}

		public void setArrayStartString(String s) {
			arrayStartString = s;
		}

		public String getArrayStartString() {
			return arrayStartString;
		}

		public void setArrayEndString(String s) {
			arrayEndString = s;
		}

		public String getArrayEndString() {
			return arrayEndString;
		}

		public void setDeliminator(String s) {
			deliminator = s;
		}

		public String getDeliminator() {
			return deliminator;
		}

		public static void ASSERTION(boolean b)
				throws StringNotInArrayFormException {
			ASSERTION(b, null);
		}

		public static void ASSERTION(boolean b, String msg)
				throws StringNotInArrayFormException {
			if (!b)
				throw new StringNotInArrayFormException(msg);
		}

		public Object parse(Class<?> elementType, String data)
				throws StringNotInArrayFormException, IllegalAccessException,
				InstantiationException, ClassNotFoundException {
			if (!isArrayForm(data)) {
				return getValueInType(elementType, data);
			}

			String[] splits = split(data);
			Object[] returnValue = new Object[splits.length];

			for (int i = 0; i < splits.length; i++) {
				returnValue[i] = parse(elementType, splits[i]);
			}
			return returnValue;
		}

		private String[] split(String data)
				throws StringNotInArrayFormException {
			ArrayList<String> returnValue = new ArrayList<String>();
			int index = 1;
			while (index < data.length()) {
				int startMarkIndex = data.indexOf(arrayStartString, index);
				int endMarkIndex = data.indexOf(arrayEndString, index);

				if (startMarkIndex < 0) {
					ASSERTION(endMarkIndex == data.length() - 1, data);
					int delimIndex = data.indexOf(deliminator, index);
					if (delimIndex < 0)
						break;
					returnValue.add(data.substring(index, delimIndex).trim());
					index = delimIndex + 1;
				} else {
					ASSERTION(startMarkIndex < endMarkIndex);
					index = endMarkIndex + 1;
					returnValue.add(data.substring(startMarkIndex, index));

				}
			}
			return returnValue.toArray(new String[1]);
		}

		public boolean isArrayForm(String data) {
			int arrayStartIndex = data.indexOf(arrayStartString);
			int arrayEndIndex = data.lastIndexOf(arrayEndString);
			return (arrayStartIndex >= 0) && (arrayEndIndex > arrayStartIndex);
		}

		public static class StringNotInArrayFormException extends
				RuntimeException {
			private static final long serialVersionUID = 1L;

			public StringNotInArrayFormException() {
				super();
			}

			public StringNotInArrayFormException(String message) {
				super(message);
			}

			public StringNotInArrayFormException(String message, Throwable cause) {
				super(message, cause);
			}

			public StringNotInArrayFormException(Throwable cause) {
				super(cause);
			}
		};

	}

	public static String getClassNameWithoutParent(Object o) {
		String className = o.getClass().getName();
		int lastDotIndex = className.lastIndexOf('.');
		int lastDollarIndex = className.lastIndexOf('$');
		int cutIndex = Math.max(lastDotIndex, lastDollarIndex) + 1;
		if (className.length() <= cutIndex) {
			return "";
		}
		return className.substring(cutIndex);
	}
	
	public static String JAR_ENTRY_FILE_SEPERATOR = "/";
	
	public static List<String> getClassNameList(File jarFile) throws IOException {
		ArrayList<String> classNameList = new ArrayList<String>();
		JarInputStream jarInputStream = new JarInputStream(
				new FileInputStream(jarFile));
		JarEntry jarEntry;
		while (true) {
			jarEntry = jarInputStream.getNextJarEntry();
			if (jarEntry == null) {
				break;
			}
			if (jarEntry.getName().endsWith(".class")) {
				classNameList.add(jarEntry.getName().replaceAll(JAR_ENTRY_FILE_SEPERATOR, "\\."));
			}
		}

		return classNameList;
	}

	/**
	 * @param args
	 */
	public static Class<?> getClass(File jarFile, String name) throws Exception {
		addURL(jarFile.toURI().toURL());

		URLClassLoader clazzLoader;
		Class<?> clazz;
		String filePath = jarFile.getAbsolutePath();
		filePath = "file:///" + filePath + "";
		URL url = new URL(filePath);
		clazzLoader = new URLClassLoader(new URL[] { url });
		clazz = clazzLoader.loadClass(name);
		return clazz;
	}

	private static void addURL(URL u) throws IOException {
		URLClassLoader sysLoader = (URLClassLoader) ClassLoader
				.getSystemClassLoader();
		URL urls[] = sysLoader.getURLs();
		for (int i = 0; i < urls.length; i++) {
			if (urls[i].toString().equalsIgnoreCase(u.toString())) {
				return;
			}
		}
		Class<?> sysclass = URLClassLoader.class;
		try {
			Method method = sysclass.getDeclaredMethod("addURL", new Class<?>[] { URL.class });
			method.setAccessible(true);
			method.invoke(sysLoader, new Object[] { u });
		} catch (Throwable t) {
			t.printStackTrace();
			throw new IOException("Error, could not add '" + u
					+ "' to system classloader");
		}
	}
	
	public static boolean isPreviousObjectInstanceOfRearObjectClass(Object o1, Object o2){
		Class<?> o2Class = o2.getClass();
		return o2Class.isInstance(o1);
	}
}

class TestDummy {
	private double scale;

	public TestDummy() {
		scale = 0;
	}

	public double getScale() {
		return scale;
	}

	public void setScale(double scale) {
		this.scale = scale;
	}

	public String toString() {
		return "scale: " + scale;
	}
};
