package com.yerihyo.yeritools.collections;


import java.util.Collection;
import java.util.Comparator;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import com.yerihyo.yeritools.math.StatisticsToolkit;

public class CountMap<K> extends HashMap<K,Integer> {
	public static final long serialVersionUID = 1;
	
	public CountMap(){ super(); }
	public CountMap(int initialCapacity){ super(initialCapacity); }
	public CountMap(int initialCapacity, float loadFactor){ super(initialCapacity, loadFactor); }
	public CountMap(Map<K,Integer> map){ super(map); }
	
	public Set<K> getKeySet(int countThreshold){
		Set<K> originalKeySet = this.keySet();
		Set<K> returnSet = new HashSet<K>();
		for(K k : originalKeySet){
			if(this.getCount(k)>=countThreshold){
				returnSet.add(k);
			}
		}
		return returnSet;
	}
	
	
	protected class CountMapEntryComparator implements Comparator<Map.Entry<K,Integer>>{
		public int compare(Map.Entry<K,Integer> e1, Map.Entry<K,Integer> e2){
			return e1.getValue()-e2.getValue();
		}		
	};
	
	public Map<K,Double> getProportionMap(){
		Map<K,Double> returnMap = new HashMap<K,Double>();
		int totalCount = this.getTotalCount();
		
		for(Map.Entry<K, Integer> entry : this.entrySet()){
			returnMap.put(entry.getKey(), ((double)entry.getValue())/totalCount);
		}
		return returnMap;
	}
	
	public static <K> Map<K,Double> getProportionMap(Map<K,Integer> countMap){
		Map<K,Double> returnMap = new HashMap<K,Double>();
		int totalCount = getTotalCount(countMap);
		
		for(Map.Entry<K, Integer> entry : countMap.entrySet()){
			returnMap.put(entry.getKey(), ((double)entry.getValue())/totalCount);
		}
		return returnMap;
	}
	
	public int getTotalCount(){
		return getTotalCount(this);
	}
	public static <K> int getTotalCount(Map<K,Integer> countMap){
		int totalCount = 0;
		for(Map.Entry<K, Integer> entry : countMap.entrySet()){
			totalCount += entry.getValue();
		}
		return totalCount;
	}
	
	public void increCount(K k){ addCount(k,1); }
	public void addCount(Map.Entry<K,Integer> entry){
		this.addCount(entry.getKey(), entry.getValue());
	}
	public void addCount(K k, int i){
		int prevCount = 0;
		if(this.containsKey(k)){
			prevCount = this.get(k);
			this.remove(k);
		}
		this.put(k, prevCount+i);
	}
	
	public void increCountAll(K[] c){
		for(K k : c){ increCount(k); }
	}
	public void increCountAll(Collection<? extends K> c){
		for(K k : c){ increCount(k); }
	}
	public void addCountAll(Map<? extends K, Integer> map){
		for(Map.Entry<? extends K,Integer> entry : map.entrySet()){
			K key = entry.getKey();
			int i = entry.getValue();
			this.addCount(key,i);			
		}
	}
	
	public void decreCount(K k){
		if(!this.containsKey(k) || (this.get(k)<=0) ){
			throw new NullPointerException("Key, such as '"+k+"' doesn't exist!");
		}
		
		int newCount = this.getCount(k)-1;
		this.remove(k);		 
		
		if(newCount<=0){ return; }	// keep the key removed
		else{ this.put(k, newCount); }		
	}
	
	public int getCount(K k){ return this.get(k); }
	
	public double entropy(){
		return StatisticsToolkit.getEntropy(getProportionMap().values().toArray(new Double[0]));
	}
}