package com.yerihyo.yeritools.io;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.IOException;
import java.io.Reader;
import java.io.Writer;
import java.util.Arrays;

import com.yerihyo.yeritools.text.StringToolkit;

public class IOToolkit {
	public static void main(String[] args) throws IOException{
		test01();
	}
	
	public static boolean hasSameContent(Reader reader1, Reader reader2) throws IOException{
		BufferedReader r1 = new BufferedReader(reader1);
		BufferedReader r2 = new BufferedReader(reader2);
		
		char[] buffer1 = new char[1024];
		char[] buffer2 = new char[1024];
		
		for(int readLength1=0, readLength2=0; (readLength1=r1.read(buffer1))>0; ){
			readLength2 = r2.read(buffer2);
			if(readLength1!=readLength2){ return false; }
			
			if(!Arrays.equals(buffer1, buffer2)){ return false; }
		}
		return true;
	}
	
	private static void test01() throws IOException {
		StringBuilder builder = new StringBuilder("hello my mom is not at home!");
		CharSequenceReader csReader = new CharSequenceReader(builder);
		
		char[] buffer = new char[1024];
		for(int readSize=0; (readSize=csReader.read(buffer))>0; ){
			System.out.println(Arrays.copyOfRange(buffer, 0, readSize));
		}
	}
	public static class CharSequenceReader extends Reader{
		private CharSequence cs;
		private int csIndex=0;
		
		public CharSequenceReader(CharSequence cs){
			this.cs = cs;
		}

		@Override
		public void close() throws IOException {
		}

		@Override
		public int read(char[] cbuf, int off, int len) throws IOException {
			if(cs.length()<=csIndex){
				return -1;
			}
				
			int csReaderAvailLength = getAvailableCharLength();
			int availLength = Math.min(len, csReaderAvailLength);
			for(int i=0; i<availLength; i++, csIndex++){
				cbuf[i+off] = cs.charAt(csIndex);
			}
			return availLength;
		}
		protected int getAvailableCharLength(){
			return cs.length() - csIndex;
		}
	}
	public static class CharSequenceWriter extends Writer{

		public CharSequenceWriter(Writer writer){
			this.writer = new BufferedWriter(writer);
		}
		private Writer writer;
		
		@Override
		public void close() throws IOException {
			writer.close();
		}

		@Override
		public void flush() throws IOException {
			writer.flush();
		}

		public void write(CharSequence cs) throws IOException{
			for(int i=0; i<cs.length(); i++){
				writer.write(cs.charAt(i));
			}
		}
		public void writeln(CharSequence cs) throws IOException{
			this.write(cs);
			writer.write(StringToolkit.newLine());
		}
		
		@Override
		public void write(char[] cbuf, int off, int len) throws IOException {
			writer.write(cbuf, off, len);
		}
		
	}
}
