package plugins.wrappers;

import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JOptionPane;

import edu.cmu.side.control.BuildModelControl;
import edu.cmu.side.control.GenesisControl;
import edu.cmu.side.model.RecipeManager.Stage;
import edu.cmu.side.model.StatusUpdater;
import edu.cmu.side.model.data.FeatureTable;
import edu.cmu.side.model.data.PredictionResult;
import edu.cmu.side.model.feature.Feature;
import edu.cmu.side.model.feature.Feature.Type;
import edu.cmu.side.model.feature.FeatureHit;
import edu.cmu.side.plugin.WrapperPlugin;
import edu.cmu.side.view.util.DefaultMap;
import edu.cmu.side.view.util.Refreshable;

public class TFIDFWrapper extends WrapperPlugin implements Refreshable
{

	private static final String FREQ_JOINER = "\tFreq:\t";
	private static final String LABEL_JOINER = "\tLabel:\t";
	private static final String FEATURE_JOINER = "\tDoc Frequency:\t";
	private static final String FEATURE_SPLIT = "\tFeature:\t";
	JCheckBox activeCbx = new JCheckBox("TF-IDF");
	boolean active = false;
	int limit = -1;

	Map<Feature, Map<String, Double>> frequencies = new HashMap<Feature, Map<String, Double>>();
	protected JButton warningButton = new JButton();

	public TFIDFWrapper()
	{
		// super();
		// selector.setEvaluator(new ChiSquaredAttributeEval());
		// TODO: replace with WarningButton utility class.
		warningButton.setIcon(new ImageIcon("toolkits/icons/error.png"));
		warningButton.setBorderPainted(false);
		warningButton.setOpaque(false);
		warningButton.setContentAreaFilled(false);
		warningButton.setVisible(false);

		warningButton.addActionListener(new ActionListener()
		{

			@Override
			public void actionPerformed(ActionEvent e)
			{
				JOptionPane.showMessageDialog(panel, warningButton.getToolTipText(), "Warning", JOptionPane.WARNING_MESSAGE);
			}
		});

		activeCbx.addItemListener(new ItemListener()
		{

			@Override
			public void itemStateChanged(ItemEvent arg0)
			{
				boolean clicked = ((JCheckBox) arg0.getSource()).isSelected();
				active = clicked;
				BuildModelControl.getWrapperPlugins().put(TFIDFWrapper.this, clicked);
				verify();
			}
		});

		panel.add("left", activeCbx);
		panel.add("left", warningButton);

		GenesisControl.addListenerToMap(Stage.FEATURE_TABLE, this);
	}

	public boolean verify()
	{
		if (!active || BuildModelControl.getHighlightedFeatureTableRecipe() == null)
		{
			warningButton.setVisible(false);
			return true;
		}
		try
		{
			if (BuildModelControl.hasHighlightedFeatureTableRecipe())
			{
				return true;
			}
			else
			{
				warningButton.setVisible(false);
			}
		}
		catch (NumberFormatException e)
		{
			warningButton.setToolTipText("Input a valid number");
		}

		warningButton.setVisible(true);
		return false;
	}

	@Override
	synchronized public void learnFromTrainingData(FeatureTable train, int fold, Map<Integer, Integer> foldsMap, StatusUpdater update)
	{
		frequencies.clear();

		// String[] labels = train.getLabelArray();
		List<String> gold = train.getAnnotations();
		
		Map<String, Double> labelCounts = new DefaultMap<String, Double>(0.0);
		
		for(String bling : gold)
		{
			labelCounts.put(bling, labelCounts.get(bling)+1.0);
		}
		
		try
		{
			for (Feature f : train.getFeatureSet())
			{
				double docFreq = 0;
				Map<String, Double> classTF = new DefaultMap<String, Double>(0.0);
				for (FeatureHit h : train.getHitsForFeature(f))
				{
					int i = h.getDocumentIndex();
					if (foldsMap.get(i) != fold)
					{
						
						String label = gold.get(i);
						double count = labelCounts.get(label);
						double value = 0;
						
						if (f.getFeatureType() == Type.BOOLEAN)
						{
							value = 1.0;
						}
						else if (f.getFeatureType() == Type.NUMERIC)
						{
							value = (Double)h.getValue();
						}
						docFreq += value;
						classTF.put(label, classTF.get(label) + value);
						
					}
					for(String label : classTF.keySet())
					{
						classTF.put(label, docFreq/classTF.get(label));
					}
				}
				//System.out.println(f+"\t"+classTF);
				frequencies.put(f, classTF); // TODO - normalize?
			}
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}

		System.out.println("TFIDF Wrapper: learned "+frequencies.size() + " feature frequencies");
		
		// System.out.println("FS 205: selected " + selectedFeatures.size() +
		// " features from table " + train.getName() + ", fold " + fold);
		// System.out.println(StringUtils.join(selectedFeatures, "\n"));

	}

	@Override
	 public FeatureTable wrapTableForSubclass(FeatureTable table, int fold, Map<Integer, Integer> foldsMap, StatusUpdater update)
	{
		Collection<FeatureHit> newHits = new HashSet<FeatureHit>();

		Set<Feature> featureSet = table.getFeatureSet();

		String[] labels = table.getLabelArray();

		for (Feature f : frequencies.keySet())
		{
			if (featureSet.contains(f))
			{
				for (int i = 0; i < labels.length; i++)
				{
					Feature f2 = Feature.fetchFeature("tfidf", f.getFeatureName() + "__tfidf__" + labels[i], Type.NUMERIC, f.getExtractor());

					for (FeatureHit hit : table.getHitsForFeature(f))
					{
						
						double value = (f.getFeatureType() == Type.BOOLEAN)?1.0:(Double)hit.getValue();
						double tfidf = value*frequencies.get(f).get(labels[i]);
						double logged = tfidf == 0? 0 : Math.log(tfidf);
						//TODO: log, normalize?
						FeatureHit hoo = new FeatureHit(f2, logged, hit.getDocumentIndex());
						newHits.add(hoo);
						//System.out.println(hoo);
					}
				}
			}
			
		}
		
		//FeatureTable newFeatureTable = table.clone();
		//newFeatureTable.addFeatureHits(newHits);
		FeatureTable newFeatureTable = new FeatureTable(table.getDocumentList(), newHits, 0, table.getAnnotation(), table.getClassValueType(), table.getLabelArray());

//		System.out.println(newFeatureTable.getHitsForDocument(0));
		
		//System.out.println(newFeatureTable.getFeatureSet());
		
//		System.out.println("TFIDF Wrapper: old = "+table.getFeatureSet().size()+", new = "+newFeatureTable.getFeatureSet().size());

		// newFeatureTable.reconcileFeatures(selectedFeatures);

		// System.out.println("FS 210: old = "+table.getFeatureSet().size()+", reconciled = "+newFeatureTable.getFeatureSet().size());

		newFeatureTable.setName("TF-IDF(" + table.getName() + "), fold=" + fold);
		return newFeatureTable;

	}

	@Override
	public PredictionResult wrapResultForSubclass(PredictionResult result, int fold, Map<Integer, Integer> foldsMap, StatusUpdater update)
	{
		return result;
	}

	@Override
	public String getOutputName()
	{
		return "tfidf";
	}

	@Override
	public String toString()
	{
		return "TF-IDF Conversion";
	}

	@Override
	protected Component getConfigurationUIForSubclass()
	{
		if (BuildModelControl.hasHighlightedFeatureTableRecipe())
		{
			Feature.Type activeClassType = BuildModelControl.getHighlightedFeatureTableRecipe().getTrainingTable().getClassValueType();
			switch (activeClassType)
			{
				case NOMINAL:
				case BOOLEAN:
				case STRING:
					activeCbx.setSelected(active);
					activeCbx.setEnabled(true);
					break;
				case NUMERIC:
					activeCbx.setSelected(false);
					activeCbx.setEnabled(false);
					break;
			}
		}
		return panel;
	}

	@Override
	public Map<String, String> generateConfigurationSettings()
	{
		Map<String, String> settings = new HashMap<String, String>();
		settings.put("active", "" + activeCbx.isSelected());
		settings.put("num", "" + limit);

		StringBuilder featureBuilder = new StringBuilder();

		for (Feature f : frequencies.keySet())
		{
			Map<String, Double> featureFreqs = frequencies.get(f);
			for (String label : featureFreqs.keySet())
			{
				featureBuilder.append(LABEL_JOINER);
				featureBuilder.append(label);
				featureBuilder.append(FREQ_JOINER);
				featureBuilder.append(featureFreqs.get(label));
			}
			featureBuilder.append(FEATURE_SPLIT);
			featureBuilder.append(f.encode());
			featureBuilder.append(FEATURE_JOINER);
		}
		settings.put("features", featureBuilder.toString());

		return settings;
	}

	@Override
	public void configureFromSettings(Map<String, String> settings)
	{
		active = settings.containsKey("active") && Boolean.TRUE.toString().equals(settings.get("active"));
		// if (active)
		{
			try
			{
				limit = Integer.parseInt(settings.get("num"));
			}
			catch (Exception e)
			{
				limit = -1;
			}
			if (settings.containsKey("features"))
			{
				frequencies.clear();
				for (String whole : settings.get("features").split(FEATURE_JOINER))
				{
					if (!whole.isEmpty()) try
					{
						String[] chunks = whole.split(FEATURE_SPLIT);
						Feature f = Feature.fetchFeature(chunks[1]);
						Map<String, Double> df = new DefaultMap<String, Double>(0.0);
						chunks = chunks[0].split(LABEL_JOINER);

						for (String chunk : chunks)
						{
							if (!chunk.isEmpty())
							{
								String[] pair = chunk.split(FREQ_JOINER);
								String label = pair[0];
								Double freq = Double.parseDouble(pair[1]);
								df.put(label, freq);
							}
						}
						frequencies.put(f, df);
					}
					catch (IllegalStateException e)
					{
						e.printStackTrace();
					}
				}
			}
		}
	}

	@Override
	public void refreshPanel()
	{
		verify();
	}

}
