package tools;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import edu.cmu.side.util.TokenizingTools;
import edu.stanford.nlp.ling.CoreLabel;
import edu.stanford.nlp.ling.Word;
import edu.stanford.nlp.parser.lexparser.LexicalizedParser;
import edu.stanford.nlp.trees.GrammaticalStructureFactory;
import edu.stanford.nlp.trees.PennTreebankLanguagePack;
import edu.stanford.nlp.trees.Tree;
import edu.stanford.nlp.trees.TreebankLanguagePack;

public class SyntacticSegmenter
{

	private static final String PARSER_MODEL_PATH = "toolkits/parser/englishPCFG.caseless.ser.gz";

	LexicalizedParser parser;
	GrammaticalStructureFactory structureFactory;

	public static void main(String[] args) throws IOException
	{
		String text = "I am not horrible because I try very hard. " + "I think you are a beautiful flower that I see."
				+ " No pants wear well but I am sleepy. " + "Oats, peas, beans, and barley grow, or do they die?";

		BufferedReader input = new BufferedReader(new FileReader("data/sentiment_documents.csv"));

		SyntacticSegmenter segmenter = new SyntacticSegmenter();
		
		String line;
		while ((line = input.readLine()) != null)
		{
			List<String> segments = segmenter.segmentLine(line, SegmentationLevel.CLAUSE);
			for (String segment : segments)
			{
				System.out.println(segment);
			}
		}
	}

	public static enum SegmentationLevel
	{
		SENTENCE, CLAUSE;
	}

	public List<String> segmentLine(String text, SegmentationLevel level)
	{

		//System.out.println(text);
		//System.out.println(level);

		if (parser == null)
		{
			parser = LexicalizedParser.loadModel(PARSER_MODEL_PATH);
			parser.setOptionFlags(new String[] { "-retainTmpSubcategories" });

			TreebankLanguagePack languagePack = new PennTreebankLanguagePack();
			structureFactory = languagePack.grammaticalStructureFactory();
		}

		List<String> segments = new ArrayList<String>();

		for (List<CoreLabel> sentence : TokenizingTools.splitSentences(text))
		{
			if (level == SegmentationLevel.SENTENCE)
			{
				int start = sentence.get(0).beginPosition();
				int end = sentence.get(sentence.size() - 1).endPosition();
				String sentenceText = text.substring(start, end);
				segments.add(sentenceText);
				continue;
			}

			Tree parse = parser.parse(sentence);

			segments.addAll(splitConjoinedSentence(parse, level, text));

		}
		return segments;
	}

	protected List<String> splitConjoinedSentence(Tree parse, SegmentationLevel level, String text)
	{
		String value = parse.value();
		// System.out.println(value);
		List<Word> words = parse.yieldWords();
		String sentenceText = text.substring(words.get(0).beginPosition(), words.get(words.size() - 1).endPosition());
		// System.out.println(sentenceText);
		// System.out.println("(----");

		// for(Tree child : parse.children())
		// System.out.print(child.value()+" ");
		// System.out.println();

		ArrayList<String> localSentences = new ArrayList<String>();
		boolean sentences = false;
		String conjunction = "";
		for (Tree kid : parse.children())
		{
			if (kid.value().startsWith("S")) sentences = true;
			if (kid.value().equals("CC")) conjunction = kid.getLeaves().get(0).value();

			if (!kid.isLeaf())
			{
				List<String> ss = splitConjoinedSentence(kid, level, text);
				if (!ss.isEmpty())
				{
					if (!conjunction.isEmpty() && sentences) ss.set(0, conjunction + " " + ss.get(0));
					localSentences.addAll(ss);
				}
			}
		}
		if (value.startsWith("S"))
		{
			if (conjunction.isEmpty() || !sentences)
			{
				localSentences.clear();
			}
			if (localSentences.isEmpty())
			{
				localSentences.add(sentenceText.trim());
			}
			else
			{
				// for(String s : localSentences)
				// {
				// if(sentenceText.contains(s))
				// {
				// sentenceText = sentenceText.replace(s, "");
				// }
				// }
				// localSentences.add(sentenceText);
			}
		}
		// System.out.println("----)");

		return localSentences;
	}

	protected List<String> getRulesForTree(Tree parse, SegmentationLevel level, String text)
	{
		String value = parse.value();
		System.out.println(value);
		List<Word> words = parse.yieldWords();
		String sentenceText = text.substring(words.get(0).beginPosition(), words.get(words.size() - 1).endPosition());
		System.out.println(sentenceText);

		ArrayList<String> sss = new ArrayList<String>();
		for (Tree kid : parse.children())
		{
			if (!kid.isLeaf())
			{
				List<String> ss = getRulesForTree(kid, level, text);
				sss.addAll(ss);
			}
			else
				return new ArrayList<String>();
		}
		if (value.equals("S"))
		{
			if (sss.isEmpty())
			{
				sss.add(sentenceText);
			}
			else
			{
				for (String s : sss)
				{
					if (sentenceText.contains(s))
					{
						sentenceText = sentenceText.replace(s, "");
					}
				}
				sss.add(sentenceText);
			}
		}

		return sss;
	}
}
